"use strict";
this.haketilo_exports = this.haketilo_exports || {};
this.haketilo_exports["content/repo_query_cacher.js"] = {};

(function() {
var globalThis = this.haketilo_this
{
/**
 * This file is part of Haketilo.
 *
 * Function: Caching responses from remote repositories and serving them to
 *           popup pages.
 *
 * Copyright (C) 2022 Wojtek Kosior
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * As additional permission under GNU GPL version 3 section 7, you
 * may distribute forms of that code without the copy of the GNU
 * GPL normally required by section 4, provided you include this
 * license notice and, in case of non-source distribution, a URL
 * through which recipients can access the Corresponding Source.
 * If you modify file(s) with this exception, you may extend this
 * exception to your version of the file(s), but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * As a special exception to the GPL, any HTML file which merely
 * makes function calls to this code, and for that purpose
 * includes it by reference shall be deemed a separate work for
 * copyright law purposes. If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * I, Wojtek Kosior, thereby promise not to sue for violation of this file's
 * license. Although I request that you do not make use of this code in a
 * proprietary program, I am not going to enforce this in court.
 */

const browser = haketilo_exports["common/browser.js"].browser;
const error_data_jsonifiable = haketilo_exports["common/misc.js"].error_data_jsonifiable;

/*
 * Map URLs to objects containing parsed responses, error info or promises
 * resolving to those. The use of promises helps us prevent multiple requests
 * for the same resource from starting concurrently.
 */
const cache = new Map();

async function perform_download(url) {
    if (cache.has(url))
	return cache.get(url);

    let resolve_cb;
    cache.set(url, new Promise(cb => resolve_cb = cb));

    try {
	var result = await browser.runtime.sendMessage(["CORS_bypass", {url}]);
	if (result === undefined)
	    throw new Error("Couldn't communicate with background script.");
    } catch(e) {
	return {error: error_data_jsonifiable(e)};
    }

    cache.set(url, result);
    resolve_cb(result);

    return result;
}

function on_repo_query_request([type, url], sender, respond_cb) {
    if (type !== "repo_query")
	return;

    perform_download(url).then(respond_cb);

    return true;
}

function start() {
    browser.runtime.onMessage.addListener(on_repo_query_request);
}
this.haketilo_exports["content/repo_query_cacher.js"].start = (start);
}
}).call({
    haketilo_exports: this.haketilo_exports,
    haketilo_this:    this
});
